#!/usr/bin/python

## vncconfig.py
## Simple front-end to vncconfig(1)
  
## Copyright (C) 2003 Tim Waugh <twaugh@redhat.com>
  
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
  
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
  
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import os
import sys

"""
TODO:

  Timings
rfbwait
deferUpdate
CompareFB
ClientWaitTimeMillis
IdleTimeout
BlacklistTimeout
BlacklistThreshold
"""

class VNCDialog:
    def __init__ (self):
        self.dialog = gtk.Dialog ()
        self.dialog.set_title ("Desktop sharing configuration")
        general_table = gtk.Table (4,3)
        general_desktop_label = gtk.Label ("Desktop name:")
        general_desktop_label.set_alignment (0, 0.5)
        self.desktop_name = gtk.Entry ()
        general_table.attach (general_desktop_label, 0, 1, 0, 1,
                              gtk.FILL, 0, 5, 5)
        general_table.attach (self.desktop_name, 1, 3, 0, 1, gtk.FILL, 0, 5, 5)

        self.password_cb = gtk.CheckButton ("Password protected")

        general_password_button = gtk.Button ("Set new password")
        general_password_button.connect ('clicked', self.set_new_password)
        general_table.attach (self.password_cb, 0, 2, 1, 2,
                              gtk.FILL, 0, 5, 5)
        general_table.attach (general_password_button, 2, 3, 1, 2,
                              gtk.FILL, 0, 5, 5)

        c = self.sharing_client = gtk.RadioButton (None)
        self.sharing_always = gtk.RadioButton (c)
        self.sharing_never = gtk.RadioButton (c)
        self.sharing_client.set_label ("Client specifies whether "
                                       "session is shared")
        self.sharing_always.set_label ("New clients always shared")
        self.sharing_never.set_label ("New clients never shared")
        svbox = gtk.VBox ()
        svbox.pack_start (self.sharing_client)
        svbox.pack_start (self.sharing_always)
        svbox.pack_start (self.sharing_never)
        general_table.attach (svbox, 0, 3, 3, 4, gtk.FILL, 0, 5, 5)

        self.disconnect_cb = gtk.CheckButton ("Disconnect existing clients "
                                              "for non-shared client")
        general_table.attach (self.disconnect_cb, 0, 3, 4, 5,
                              gtk.FILL, 0, 5, 5)

        notebook = gtk.Notebook ()
        notebook.append_page (general_table, gtk.Label ("General"))

        protocol_table = gtk.Table (4, 2)
        self.protocol33_cb = gtk.CheckButton ("Use version 3.3 "
                                              "(for badly-behaved clients)")
        protocol_table.attach (self.protocol33_cb, 0, 2, 0, 1,
                               gtk.FILL, 0, 5, 5)

        rfbport_label = gtk.Label ("RFB port:")
        rfbport_label.set_alignment (0, 0.5)
        self.rfbport = gtk.SpinButton ()
        self.rfbport.set_digits (0)
        self.rfbport.set_increments (1, 10)
        self.rfbport.set_range (1025, 65535)
        protocol_table.attach (rfbport_label, 0, 1, 1, 2, gtk.FILL, 0, 5, 5)
        protocol_table.attach (self.rfbport, 1, 2, 1, 2, gtk.FILL, 0, 5, 5)

        self.allow_http = gtk.CheckButton ("Allow web browser clients")
        self.allow_http.connect ('toggled', self.allow_http_toggled)
        protocol_table.attach (self.allow_http, 0, 2, 2, 3, gtk.FILL, 0, 5, 5)

        httpport_label = gtk.Label ("HTTP port:")
        httpport_label.set_alignment (0, 0.5)
        self.httpport = gtk.SpinButton ()
        self.httpport.set_digits (0)
        self.httpport.set_increments (1, 10)
        self.httpport.set_range (1025, 65535)
        space = gtk.Label ("    ")
        http_table = gtk.Table (2, 3)
        http_table.attach (space, 0, 1, 0, 1, gtk.FILL, 0, 5, 5)
        http_table.attach (httpport_label, 1, 2, 0, 1, gtk.FILL, 0, 5, 5)
        http_table.attach (self.httpport, 2, 3, 0, 1, gtk.FILL, 0, 5, 5)

        directory_label = gtk.Label ("Directory:")
        self.httpd = gtk.Entry ()
        http_table.attach (directory_label, 1, 2, 1, 2, gtk.FILL, 0, 5, 5)
        http_table.attach (self.httpd, 2, 3, 1, 2, gtk.FILL, 0, 5, 5)
        protocol_table.attach (http_table, 0, 2, 3, 4, gtk.FILL, 0, 0, 0)

        notebook.append_page (protocol_table, gtk.Label ("Protocol"))

        options_vbox = gtk.VBox ()
        self.sendcut = gtk.CheckButton ("Send cut text")
        self.acceptcut = gtk.CheckButton ("Accept cut text")
        self.acceptptr = gtk.CheckButton ("Accept mouse events")
        self.acceptkey = gtk.CheckButton ("Accept keyboard events")
        options_vbox.pack_start (self.sendcut)
        options_vbox.pack_start (self.acceptcut)
        options_vbox.pack_start (self.acceptptr)
        options_vbox.pack_start (self.acceptkey)
        notebook.append_page (options_vbox, gtk.Label ("Options"))

        self.dialog.vbox.pack_start (notebook)
        self.dialog.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.dialog.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.dialog.vbox.show_all ()

    def allow_http_toggled (self, button):
        active = button.get_active ()
        if self.orig_httpport == 0:
            self.httpport.set_value (self.rfbport.get_value () - 100)
        else:
            self.httpport.set_value (self.orig_httpport)

        if self.httpd.get_text () == "":
            self.httpd.set_text ("/usr/share/vnc/classes")

        for widget in [self.httpport, self.httpd]:
            widget.set_sensitive (active)

    def do_set_new_password (self, passwd):
        (pid, fd) = os.forkpty ()

        if not pid:
            os.execvp ("vncpasswd", [ "vncpasswd" ])
            sys.exit (1)

        os.read (fd, 1000)
        os.write (fd, passwd + '\n')
        os.read (fd, 1000)
        os.read (fd, 1000)
        os.write (fd, passwd + '\n')
        os.read (fd, 1000)

        try:
            (pid, status) = os.waitpid (pid, 0)
            value = self.get_value ("passwordFile")
            self.set_value ("passwordFile", value)
        except OSError, (errno, msg):
            print "waitpid:", msg

    def set_new_password (self, button):
        dialog = gtk.Dialog ()
        dialog.set_title ("Set password")
        dialog.set_transient_for (self.dialog)
        dialog.set_modal (gtk.TRUE)
        table = gtk.Table (2, 2)
        label1 = gtk.Label ("Password:")
        label2 = gtk.Label ("Verify:")
        label1.set_alignment (0, 0.5)
        label2.set_alignment (0, 0.5)
        table.attach (label1, 0, 1, 0, 1, gtk.FILL, 0, 5, 5)
        table.attach (label2, 0, 1, 1, 2, gtk.FILL, 0, 5, 5)
        password1 = gtk.Entry ()
        password2 = gtk.Entry ()
        password1.set_visibility (gtk.FALSE)
        password2.set_visibility (gtk.FALSE)
        table.attach (password1, 1, 2, 0, 1, gtk.FILL, 0, 5, 5)
        table.attach (password2, 1, 2, 1, 2, gtk.FILL, 0, 5, 5)
        dialog.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK)
        dialog.vbox.pack_start (table)
        dialog.vbox.show_all ()
        while 1:
            response = dialog.run ()
            if response != gtk.RESPONSE_OK:
                break;

            p1 = password1.get_text ()
            p2 = password2.get_text ()
            if len (p1) < 6 or len (p2) < 6:
                err = gtk.MessageDialog (dialog, gtk.DIALOG_MODAL,
                                         gtk.MESSAGE_ERROR,
                                         gtk.BUTTONS_OK,
                                         "Must be at least 6 characters")
                err.run ()
                err.destroy ()
                continue

            if p1 != p2:
                err = gtk.MessageDialog (dialog, gtk.DIALOG_MODAL,
                                         gtk.MESSAGE_ERROR,
                                         gtk.BUTTONS_OK,
                                         "Passwords don't match")
                err.run ()
                err.destroy ()
                continue

            self.do_set_new_password (p1)
            break
            
        dialog.destroy ()
        if response == gtk.RESPONSE_OK:
            pass

    def get_value (self, field):
        try:
            os.environ["FIELD"] = field
            output = os.popen ('vncconfig -get "$FIELD"').readlines ()
            return output[0][:len (output[0]) - 1]
        except:
            err = gtk.MessageDialog (None, gtk.DIALOG_MODAL,
                                     gtk.MESSAGE_ERROR,
                                     gtk.BUTTONS_OK,
                                     "VNC extension module not present")
            err.run ()
            sys.exit (1)

    def get_string (self, field):
        return self.get_value (field)

    def get_bool (self, field):
        value = self.get_value (field)
        if value[0].isdigit ():
            try:
                return int (value)
            except:
                return gtk.FALSE

        if value.lower () == 'true':
            return gtk.TRUE

        return gtk.FALSE

    def populate_fields (self):
        self.orig_desktop_name = self.get_value ("desktop")
        self.desktop_name.set_text (self.orig_desktop_name)
        self.orig_password_cb = self.get_string ("SecurityTypes") != "None"
        self.password_cb.set_active (self.orig_password_cb)
        self.sharing_client.set_active (gtk.TRUE)
        self.orig_sharing_always = self.get_bool ("AlwaysShared")
        self.sharing_always.set_active (self.orig_sharing_always)
        self.orig_sharing_never = self.get_bool ("NeverShared")
        self.sharing_never.set_active (self.orig_sharing_never)
        self.orig_disconnect_cb = self.get_bool ("DisconnectClients")
        self.disconnect_cb.set_active (self.orig_disconnect_cb)

        self.orig_protocol33_cb = self.get_bool ("Protocol3.3")

        try:
            self.orig_rfbport = int (self.get_string ("rfbPort"))
        except:
            self.orig_rfbport = 5900

        if not self.orig_rfbport:
            # Can't change RFB port for real X server.
            self.orig_rfbport = 5900
            self.rfbport.set_sensitive (gtk.FALSE)

        self.rfbport.set_value (self.orig_rfbport)

        try:
            self.orig_httpport = int (self.get_string ("httpPort"))
        except:
            self.orig_httpport = self.rfbport.get_value () - 100

        self.httpport.set_value (self.orig_httpport)

        self.orig_httpd = self.get_string ("httpd")
        self.httpd.set_text (self.orig_httpd)
        self.allow_http.set_active (self.orig_httpd != "")
        self.allow_http_toggled (self.allow_http)

        self.orig_sendcut = self.get_bool ("SendCutText")
        self.sendcut.set_active (self.orig_sendcut)
        self.orig_acceptcut = self.get_bool ("AcceptCutText")
        self.acceptcut.set_active (self.orig_acceptcut)
        self.orig_acceptptr = self.get_bool ("AcceptPointerEvents")
        self.acceptptr.set_active (self.orig_acceptptr)
        self.orig_acceptkey = self.get_bool ("AcceptKeyEvents")
        self.acceptkey.set_active (self.orig_acceptkey)

    def set_value (self, field, value):
        os.environ["FIELD"] = field
        os.environ["VALUE"] = str (value)
        os.system ('vncconfig "$FIELD"="$VALUE"')

    def set_bool (self, field, value):
        if value:
            self.set_value (field, 'true')
        else:
            self.set_value (field, 'false')

    def run (self, files = None):
        self.populate_fields ()
        response = self.dialog.run ()
        self.dialog.hide ()
        if response != gtk.RESPONSE_OK:
            return

        desktop_name = self.desktop_name.get_text ()
        if self.orig_desktop_name != desktop_name:
            self.set_value ("desktop", desktop_name)

        password_cb = self.password_cb.get_active ()
        if self.orig_password_cb != password_cb:
            if password_cb:
                self.set_value ("SecurityTypes", "VncAuth")
            else:
                self.set_value ("SecurityTypes", "None")

        sharing_always = self.sharing_always.get_active ()
        if self.orig_sharing_always != sharing_always:
            self.set_bool ("AlwaysShared", sharing_always)

        sharing_never = self.sharing_never.get_active ()
        if self.orig_sharing_never != sharing_never:
            self.set_bool ("NeverShared", sharing_never)

        disconnect_cb = self.disconnect_cb.get_active ()
        if self.orig_disconnect_cb != disconnect_cb:
            self.set_bool ("DisconnectClients", disconnect_cb)

        protocol33_cb = self.protocol33_cb.get_active ()
        if self.orig_protocol33_cb != protocol33_cb:
            self.set_bool ("Protocol3.3", protocol33_cb)

        rfbport = self.rfbport.get_value_as_int ()
        if self.orig_rfbport != rfbport:
            self.set_value ("rfbPort", rfbport)

        httpport = self.httpport.get_value_as_int ()
        httpd = self.httpd.get_text ()
        allow_http = self.allow_http.get_active ()
        if allow_http:
            if self.orig_httpport != httpport:
                self.set_value ("httpPort", httpport)
            if self.orig_httpd != httpd:
                self.set_value ("httpd", httpd)
        else:
            if self.orig_httpport:
                self.set_value ("httpPort", "0")
            if self.orig_httpd:
                self.set_value ("httpd", "")

        sendcut = self.sendcut.get_active ()
        if self.orig_sendcut != sendcut:
            self.set_bool ("SendCutText", sendcut)
        acceptcut = self.acceptcut.get_active ()
        if self.orig_acceptcut != acceptcut:
            self.set_bool ("AcceptCutText", acceptcut)
        acceptptr = self.acceptptr.get_active ()
        if self.orig_acceptptr != acceptptr:
            self.set_bool ("AcceptPointerEvents", acceptptr)
        acceptkey = self.acceptkey.get_active ()
        if self.orig_acceptkey != acceptkey:
            self.set_bool ("AcceptKeyEvents", acceptkey)

VNCDialog ().run ()
